; MAIN PROGRAM
;-------------


; Global declarations
;-------------------
Global gravity, deltaTime, deltaTenTime, FPS, roomN%, shot, sound, channel

Const MUSEO = 1
Const CAMERA = 2
Const SCENERY = 3
Const BALL = 5

Const AFIRE = 10
Const AAIR = 11
Const AEARTH = 12
Const AWATER = 13

FPS = 30
gravity = 1.0 

; Setting graphics
;------------------

Include "settingGraphics.bb"


; Type declarations
;------------------

Type Player
	Field entity
	Field avatar
	Field cam
	Field vel
	Field grav#
End Type

Type Ball
	Field entity
	Field mesh
	Field vel#
	Field grav#
End Type

; Setting Collisions
;-------------------

Collisions CAMERA,MUSEO,2,3
Collisions BALL,MUSEO,2,3

; Timing
;--------

period=1000/FPS
time = MilliSecs()-period

;----------

room = LoadMesh("./rooms/Museum.3ds")
EntityType room,MUSEO

AmbientLight 255,255,255
light = CreateLight()

player1.Player = createPlayer()
roomN% = 110
SoundVolume sound,0.4


; MAIN LOOP
;----------

deltaTenTime = 0

While Not KeyHit(1)	
	Repeat
		elapsed=MilliSecs()-time
	Until elapsed

	;how many 'frames' have elapsed	
	ticks=elapsed/period
	
	For k=1 To ticks
		time=time+period
		If k=ticks Then 
			CaptureWorld 
		End If
		If(k=ticks) Then 
			deltaTime = 1
			deltaTenTime = deltaTenTime + 1
			If deltaTenTime = 60 Then
				deltaTenTime = 0
			End If 
		End If 
		UpdateMuseum()

		UpdateWorld
		deltaTime = 0
	Next
	RenderWorld
	;Text 10,10,"POSITIONS " + EntityX(player1\entity, True)+ ", " + EntityY(player1\entity, True) + " , " +  EntityZ(player1\entity, True)
	;Text 10,30,"RELATIVE ANGLES" + EntityPitch(player1\entity)+ ", " + EntityYaw(player1\entity) + " , " +  EntityRoll(player1\entity, False)
	;Text 10,40,"roomN" + roomN
	;Text 10,50,"TIME" + deltaTenTime


	Flip
Wend

End

Function Updatemuseum()
	For p.Player=Each Player
		UpdatePlayer( p )
	Next
	For b.Ball = Each Ball
		UpdateBall(b)
	Next
End Function




Function CreatePlayer.Player()
	
	p.Player=New Player
	
	p\entity=CreatePivot()
	p\avatar = LoadMesh(".\avatars\Bob.3ds",p\entity)
	p\cam = CreateCamera(p\entity)
	
	ScaleMesh p\avatar,.5,.5,.5
	
	EntityRadius p\entity,MeshHeight(p\avatar)
	EntityType p\entity,CAMERA
	
	CameraRange p\cam,5,5000
	
	p\vel = 10

	HideEntity p\entity
	
	PositionEntity p\entity,0,100,400
	ShowEntity p\entity
	
	Return p
End Function

Function UpdatePlayer( p.Player )

	If EntityX(p\entity,True) > 2573 And EntityX(p\entity,True) < 5173 Then
	
		If  EntityZ(p\entity,True) > 935 And EntityZ(p\entity,True) < 1747 Then
			roomN% = 1
			
			CameraFogMode p\cam,1
			CameraFogColor p\cam,10,10,10
			
		Else If EntityZ(p\entity,True) > 41 And EntityZ(p\entity,True) < 852 Then
			roomN% = 2
			If deltaTenTime = 0 Then
				If(Rand Mod 2 = 0) Then
					sound = LoadSound(".\sounds\bird1.wav")
				Else 
					sound = LoadSound(".\sounds\bird2.wav")
				End If 
				channel = PlaySound(sound)
				ResumeChannel channel 
			End If
			
			CameraFogMode p\cam,1
			CameraFogColor p\cam,190,190,190
			
		Else If EntityZ(p\entity,True) > -852  And EntityZ(p\entity,True) < -41 Then
			roomN% = 3
			If deltaTenTime = 0 Then
				sound = LoadSound(".\sounds\wind1.wav")
				channel = PlaySound(sound)
				ResumeChannel channel 
			End If
			
			CameraFogMode p\cam,0
			
		Else If EntityZ(p\entity,True) > -1747 And EntityZ(p\entity,True) < -935 Then
			roomN% = 4
			If deltaTenTime = 0 Then
				sound = LoadSound(".\sounds\sonar.wav")
				SoundVolume sound,0.2
				channel = PlaySound(sound)
				ResumeChannel channel 
			End If
			
			CameraFogMode p\cam,1
			CameraFogColor p\cam,10,10,255
			
		End If
	Else
		roomN% = 110
		PauseChannel channel
		
		CameraFogMode p\cam,0
		
	End If
	
	; Movements
	If KeyDown(203)
		TurnEntity p\entity,0,5,0
	Else If KeyDown(205)
		TurnEntity p\entity,0,-5,0
	EndIf
	If KeyDown(200)		
		MoveEntity p\entity,0,0,p\vel
	Else If KeyDown(208)	
		MoveEntity p\entity,0,0,-p\vel
	EndIf
	
	If KeyHit(44) ;jump/fly
		p\grav# = 10.0
		MoveEntity p\entity,0,p\grav,0
	End If
	
	
	If KeyDown(46)
		TurnEntity p\cam,1.0,0,0
	Else If KeyDown(47)	
		TurnEntity p\cam,-1.0,0,0
	EndIf
	
	If KeyDown(30)	
		MoveEntity p\cam,0,0,-p\vel
	Else If KeyDown(32)
		MoveEntity p\cam,0,0,p\vel
	EndIf
	
	If KeyHit(16) Then ; shot
		CreateBall( EntityX(p\entity,True), EntityY(p\entity,True), EntityZ(p\entity,True),EntityPitch(p\cam),EntityYaw(p\entity),EntityRoll(p\entity))
	End If
	
	;Gravity
	If deltaTime = 1 Then
				 ; a second is gone
				If roomN% = 3 Then ;air
					p\grav# = p\grav# + gravity
					p\grav# = p\grav# * 0.5
				Else If roomN% = 4 Then ;water
					p\grav# = p\grav# * 0.1
				Else p\grav# = p\grav# - gravity
				End If
				MoveEntity p\entity,0,p\grav#,0
			End If 
End Function

; ----------------------------

Function CreateBall(x,y,z,pitch,yaw,roll)
	
	
	b.Ball=New Ball
	b\entity = CreatePivot()
	
	If roomN% = 1 Then
		b\mesh = LoadMesh(".\objects\fireBall.3ds",b\entity)
		shot = LoadSound(".\sounds\gun1.wav")
		SoundVolume shot,1
		PlaySound(shot)

	Else If roomN% = 2 Then
		b\mesh = LoadMesh(".\objects\earthBall.3ds",b\entity)
		shot = LoadSound(".\sounds\frog.wav")
		SoundVolume shot,1
		PlaySound(shot)

	Else If roomN% = 3 Then
		b\mesh = LoadMesh(".\objects\airBall.3ds",b\entity)
		shot = LoadSound(".\sounds\windTube.wav")
		SoundVolume shot,0.5
		PlaySound(shot)

	Else If roomN% = 4 Then
		b\mesh = LoadMesh(".\objects\waterBall.3ds",b\entity)
		shot = LoadSound(".\sounds\bubbles.wav")
		SoundVolume shot,1
		PlaySound(shot)

	Else
		b\mesh = LoadMesh(".\objects\palla.3ds",b\entity)
		shot = LoadSound(".\sounds\boing.wav")
		SoundVolume shot,1
		PlaySound(shot)
	End If

	ScaleMesh b\mesh,0.1,0.1,0.1
	
	EntityRadius b\entity,MeshWidth(b\mesh)-2

	PositionEntity b\entity,x,y+10,z

	RotateEntity b\entity,pitch,yaw,roll

	b\vel#=30
	
	b\grav# = 0.0

	MoveEntity b\entity,0,b\grav,b\vel

	
	EntityType b\entity,BALL
End Function

Function UpdateBall(b.Ball)
	; Acc = ( m * g - k*v/m)
	
	;acc# = (1.0 * gravity - 1.0 * b\grav# / 1.0)
	
	If(b\vel# > 0.5 And Abs(b\vel#) < 1000.0) Then 	
			If EntityCollided(b\entity,MUSEO)  Then

				x# = CollisionX(b\entity, 1)
				y# = CollisionY(b\entity, 1)
				z# = CollisionZ(b\entity, 1)

				nx# = CollisionNX(b\entity, 1)
				ny# = CollisionNY(b\entity, 1)
				nz# = CollisionNZ(b\entity, 1)

				AlignToVector b\entity,nx#,0,nz#,3
			
				b\vel# = b\vel# * 0.8
			
				; Qui hai cambiato...
				If (ny# <> 0.0 ) Then
					b\grav# = - b\grav# * 0.8	
				If roomN% = 2 Then
					cube = CreateCube()
	
					PositionEntity cube,x,y,z
					ScaleEntity cube,4,4,4
					
					texture = LoadTexture(".\sprites\fiore.BMP")
					EntityTexture cube,texture
					
					EntityType cube,MUSEO
				End If 
				
					
				End If
				
				
			End If	
			If deltaTime = 1 Then
				 ; a second is gone
				If roomN% = 3 Then ;air
					b\grav# = b\grav# + gravity
					b\grav# = b\grav# * 0.5
				Else If roomN% = 4 Then ;water
					b\grav# = b\grav# * 0.1
				Else b\grav# = b\grav# - gravity
				End If
				TranslateEntity b\entity,0,b\grav#,0
			End If 
			MoveEntity b\entity,0,0,b\vel
	Else 
		
		
		ResetEntity b\entity
		FreeEntity b\mesh
		FreeEntity b\entity
		Delete b
	End If
End Function